"""
pytest config for sphinxcontrib/apidoc/tests
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

:copyright: Copyright 2018-present by Stephen Finucane <stephen@that.guru>
:license: BSD, see LICENSE for details.
"""

import os
import tempfile

import pytest
import sphinx

pytest_plugins = 'sphinx.testing.fixtures'

collect_ignore = ['roots']


@pytest.fixture(scope='session')
def sphinx_test_tempdir():
    if sphinx.version_info >= (7, 2, 0):
        from pathlib import Path

        return Path(
            os.environ.get('SPHINX_TEST_TEMPDIR', tempfile.mkdtemp(prefix='apidoc-'))
        ).resolve()
    else:
        from sphinx.testing.path import path

        return path(
            os.environ.get('SPHINX_TEST_TEMPDIR', tempfile.mkdtemp(prefix='apidoc-'))
        ).abspath()


@pytest.fixture(scope='session')
def rootdir():
    if sphinx.version_info >= (7, 2, 0):
        from pathlib import Path

        return Path(os.path.dirname(__file__) or '.').resolve() / 'roots'
    else:
        from sphinx.testing.path import path

        return path(os.path.dirname(__file__) or '.').abspath() / 'roots'



import os
import sys

sys.path.insert(0, os.path.abspath('./'))

extensions = ['sphinxcontrib.apidoc']
master_doc = 'index'

apidoc_module_dir = '.'
apidoc_excluded_paths = ['conf.py']
apidoc_separate_modules = True
apidoc_toc_file = 'custom'
apidoc_module_first = True
apidoc_extra_args = ['--private']



"""
sphinxcontrib.apidoc
~~~~~~~~~~~~~~~~~~~~

A Sphinx extension for running 'sphinx-apidoc' on each build.

:copyright: Copyright 2018-present by Stephen Finucane <stephen@that.guru>
:license: BSD, see LICENSE for details.
"""

import pbr.version
from typing import Any

from sphinx.application import Sphinx
from sphinxcontrib.apidoc import ext

__version__ = pbr.version.VersionInfo('sphinxcontrib-apidoc').version_string()


def setup(app: Sphinx) -> dict[str, Any]:
    app.setup_extension('sphinx.ext.autodoc')  # We need autodoc to function

    app.connect('builder-inited', ext.builder_inited)
    app.add_config_value('apidoc_module_dir', None, 'env', [str])
    app.add_config_value('apidoc_output_dir', 'api', 'env', [str])
    app.add_config_value('apidoc_template_dir', 'templates', 'env', [str])
    app.add_config_value('apidoc_excluded_paths', [], 'env', list[str])
    app.add_config_value('apidoc_separate_modules', False, 'env', [bool])
    app.add_config_value('apidoc_toc_file', None, 'env', [str, bool])
    app.add_config_value('apidoc_module_first', False, 'env', [bool])
    app.add_config_value('apidoc_extra_args', [], 'env', [list])

    return {'version': __version__, 'parallel_read_safe': True}



"""
sphinxcontrib.apidoc.ext
~~~~~~~~~~~~~~~~~~~~~~~~

A Sphinx extension for running 'sphinx-apidoc' on each build.

:copyright: Copyright 2018-present by Stephen Finucane <stephen@that.guru>
:license: BSD, see LICENSE for details.
"""

from os import path

from sphinx.application import Sphinx
from sphinx.ext import apidoc
from sphinx.util import logging

logger = logging.getLogger(__name__)


def builder_inited(app: Sphinx) -> None:
    module_dir = app.config.apidoc_module_dir
    output_dir = path.join(app.srcdir, app.config.apidoc_output_dir)
    template_dir = path.join(app.srcdir, app.config.apidoc_template_dir)
    excludes = app.config.apidoc_excluded_paths
    separate_modules = app.config.apidoc_separate_modules
    toc_file = app.config.apidoc_toc_file
    module_first = app.config.apidoc_module_first
    extra_args = app.config.apidoc_extra_args

    if not module_dir:
        logger.warning("No 'apidoc_module_dir' specified; skipping API doc generation")
        return

    # if the path is relative, make it relative to the 'conf.py' directory
    if not path.isabs(module_dir):
        module_dir = path.abspath(path.join(app.srcdir, module_dir))

    if not path.exists(module_dir):
        logger.warning(
            "The path defined in 'apidoc_module_dir' does not "
            "exist; skipping API doc generation; %s",
            module_dir,
        )
        return

    # refactor this module so that we can call 'recurse_tree' like a sane
    # person - at present there is way too much passing around of the
    # 'optparse.Value' instance returned by 'optparse.parse_args'
    def cmd_opts():
        yield '--force'

        if separate_modules:
            yield '--separate'

        if isinstance(toc_file, bool) and toc_file is False:
            yield '--no-toc'
        elif toc_file:
            yield '--tocfile'
            yield toc_file

        if module_first:
            yield '--module-first'

        yield '--output-dir'
        yield output_dir

        yield '--templatedir'
        yield template_dir

        yield from extra_args

        yield module_dir

        for exc in excludes:
            yield path.abspath(path.join(module_dir, exc))

    apidoc.main(list(cmd_opts()))
